@@ -2,8 +2,8 @@
 # -*- coding: utf-8 -*-
 
 from io import BytesIO, StringIO
-from typing import Optional, Dict, Any, List
-from urllib.parse import urljoin
+from typing import Optional, Dict, Any, List, Union
+from urllib.parse import urljoin, urlparse
 from pathlib import Path
 
 import requests
@@ -25,6 +25,9 @@ class Lookyloo():
         :param root_url: URL of the instance to query.
         '''
         self.root_url = root_url
+
+        if not urlparse(self.root_url).scheme:
+            self.root_url = 'http://' + self.root_url
         if not self.root_url.endswith('/'):
             self.root_url += '/'
         self.session = requests.session()
@@ -36,6 +39,26 @@ class Lookyloo():
         r = self.session.head(self.root_url)
         return r.status_code == 200
 
+    def get_status(self, tree_uuid: str) -> Dict[str, Any]:
+        '''Get the status of a capture:
+            * -1: Unknown capture.
+            * 0: The capture is queued up but not processed yet.
+            * 1: The capture is ready.
+            * 2: The capture is ongoing and will be ready soon.
+        '''
+        r = self.session.get(urljoin(self.root_url, str(Path('json', tree_uuid, 'status'))))
+        return r.json()
+
+    def get_capture_stats(self, tree_uuid: str) -> Dict[str, Any]:
+        '''Get statistics of the capture'''
+        r = self.session.get(urljoin(self.root_url, str(Path('json', tree_uuid, 'stats'))))
+        return r.json()
+
+    def get_info(self, tree_uuid: str) -> Dict[str, Any]:
+        '''Get information about the capture (url, timestamp, user agent)'''
+        r = self.session.get(urljoin(self.root_url, str(Path('json', tree_uuid, 'info'))))
+        return r.json()
+
     def enqueue(self, url: Optional[str]=None, quiet: bool=False, **kwargs) -> str:
         '''Enqueue an URL.
 
@@ -51,18 +74,20 @@ class Lookyloo():
         else:
             to_send = kwargs
         response = self.session.post(urljoin(self.root_url, 'submit'), json=to_send)
+        uuid = response.json()
         if quiet:
-            return response.text
+            return uuid
         else:
-            return urljoin(self.root_url, f'tree/{response.text}')
+            return urljoin(self.root_url, f'tree/{uuid}')
 
     def get_apikey(self, username: str, password: str) -> Dict[str, str]:
+        '''Get the API key for the given user.'''
         to_post = {'username': username, 'password': password}
-        r = self.session.post(urljoin(self.root_url, str(Path('json', 'get_token'))),
-                              json=to_post)
+        r = self.session.post(urljoin(self.root_url, str(Path('json', 'get_token'))), json=to_post)
         return r.json()
 
     def init_apikey(self, username: Optional[str]=None, password: Optional[str]=None, apikey: Optional[str]=None):
+        '''Init the API key for the current session. All the requests against lookyloo after this call will be authenticated.'''
         if apikey:
             self.apikey = apikey
         elif username and password:
@@ -77,15 +102,42 @@ class Lookyloo():
             raise AuthError('Unable to initialize API key')
 
     def misp_export(self, tree_uuid: str) -> Dict:
+        '''Export the capture in MISP format'''
         r = self.session.get(urljoin(self.root_url, str(Path('json', tree_uuid, 'misp_export'))))
         return r.json()
 
-    def misp_push(self, tree_uuid: str) -> Dict:
+    def misp_push(self, tree_uuid: str) -> Union[Dict, List]:
+        '''Push the capture to a pre-configured MISP instance (requires an authenticated user, use init_apikey first)
+        Note: if the response is a dict, it is an error mesage. If it is a list, it's a list of MISP event.
+        '''
         if not self.apikey:
             raise AuthError('You need to initialize the apikey to use this method (see init_apikey)')
         r = self.session.get(urljoin(self.root_url, str(Path('json', tree_uuid, 'misp_push'))))
         return r.json()
 
+    def trigger_modules(self, tree_uuid: str, force: bool=False) -> Dict:
+        '''Trigger all the available 3rd party modules on the given capture.
+        :param force: Trigger the modules even if they were already triggered today.
+        '''
+        to_send = {'force': force}
+        r = self.session.post(urljoin(self.root_url, str(Path('json', tree_uuid, 'trigger_modules'))),
+                              json=to_send)
+        return r.json()
+
+    def rebuild_capture(self, tree_uuid: str) -> Dict:
+        '''Force rebuild a capture (requires an authenticated user, use init_apikey first)'''
+        if not self.apikey:
+            raise AuthError('You need to initialize the apikey to use this method (see init_apikey)')
+        r = self.session.get(urljoin(self.root_url, str(Path('admin', tree_uuid, 'rebuild'))))
+        return r.json()
+
+    def hide_capture(self, tree_uuid: str) -> Dict:
+        '''Hide a capture from the index page (requires an authenticated user, use init_apikey first)'''
+        if not self.apikey:
+            raise AuthError('You need to initialize the apikey to use this method (see init_apikey)')
+        r = self.session.get(urljoin(self.root_url, str(Path('admin', tree_uuid, 'hide'))))
+        return r.json()
+
     def get_redirects(self, capture_uuid: str) -> Dict[str, Any]:
         '''Returns the initial redirects.
 
@@ -94,12 +146,28 @@ class Lookyloo():
         r = self.session.get(urljoin(self.root_url, str(Path('json', capture_uuid, 'redirects'))))
         return r.json()
 
+    def get_urls(self, capture_uuid: str) -> Dict[str, Any]:
+        '''Returns all the URLs seen during the capture.
+
+        :param capture_uuid: UUID of the capture
+        '''
+        r = self.session.get(urljoin(self.root_url, str(Path('json', capture_uuid, 'urls'))))
+        return r.json()
+
+    def get_hostnames(self, capture_uuid: str) -> Dict[str, Any]:
+        '''Returns all the hostnames seen during the capture.
+
+        :param capture_uuid: UUID of the capture
+        '''
+        r = self.session.get(urljoin(self.root_url, str(Path('json', capture_uuid, 'hostnames'))))
+        return r.json()
+
     def get_screenshot(self, capture_uuid: str) -> BytesIO:
         '''Returns the screenshot.
 
         :param capture_uuid: UUID of the capture
         '''
-        r = self.session.get(urljoin(self.root_url, str(Path('tree', capture_uuid, 'image'))))
+        r = self.session.get(urljoin(self.root_url, str(Path('bin', capture_uuid, 'screenshot'))))
         return BytesIO(r.content)
 
     def get_cookies(self, capture_uuid: str) -> List[Dict[str, str]]:
@@ -107,7 +175,7 @@ class Lookyloo():
 
         :param capture_uuid: UUID of the capture
         '''
-        r = self.session.get(urljoin(self.root_url, str(Path('tree', capture_uuid, 'cookies'))))
+        r = self.session.get(urljoin(self.root_url, str(Path('json', capture_uuid, 'cookies'))))
         return r.json()
 
     def get_html(self, capture_uuid: str) -> StringIO:
@@ -118,20 +186,24 @@ class Lookyloo():
         r = self.session.get(urljoin(self.root_url, str(Path('tree', capture_uuid, 'html'))))
         return StringIO(r.text)
 
-    def get_hashes(self, capture_uuid: str) -> StringIO:
+    def get_hashes(self, capture_uuid: str, algorithm: str='sha512', hashes_only: bool=True) -> StringIO:
         '''Returns all the hashes of all the bodies (including the embedded contents)
 
         :param capture_uuid: UUID of the capture
+        :param algorithm: The algorithm of the hashes
+        :param hashes_only: If False, will also return the URLs related to the hashes
         '''
-        r = self.session.get(urljoin(self.root_url, str(Path('tree', capture_uuid, 'hashes'))))
-        return StringIO(r.text)
+        params: Dict[str, Union[str, int]] = {'algorithm': algorithm, 'hashes_only': int(hashes_only)}
+
+        r = self.session.get(urljoin(self.root_url, str(Path('json', capture_uuid, 'hashes'))), params=params)
+        return r.json()
 
     def get_complete_capture(self, capture_uuid: str) -> BytesIO:
         '''Returns a zip files that contains the screenshot, the har, the rendered HTML, and the cookies.
 
         :param capture_uuid: UUID of the capture
         '''
-        r = self.session.get(urljoin(self.root_url, str(Path('tree', capture_uuid, 'export'))))
+        r = self.session.get(urljoin(self.root_url, str(Path('bin', capture_uuid, 'export'))))
         return BytesIO(r.content)
 
     def get_hash_occurrences(self, h: str) -> Dict[str, Any]:
